import os
import sys


def is_snap():
  return bool(os.getenv('SNAP'))


def check_snap_permissions(directory: str):
  try:
    any(os.scandir(directory))
  except PermissionError:
    homedir = os.getenv('SNAP_REAL_HOME')
    if not homedir:
      print(
        'SNAP_REAL_HOME environment variable is not set.'
        ' Please, report the issue to the rclip project on'
        ' GitHub https://github.com/yurijmikhalevich/rclip/issues.'
      )
      sys.exit(1)
    if directory == homedir or directory.startswith(homedir + os.sep):
      print(
        'rclip doesn\'t have access to the current directory.'
        ' You can resolve this issue by running:'
        '\n\n\tsudo snap connect rclip:home\n\n'
        'This command will grant rclip the necessary access to the home directory.'
        ' Afterward, you can try again.'
      )
      sys.exit(1)
    if directory == '/media' or directory.startswith('/media' + os.sep):
      print(
        'rclip doesn\'t have access to the current directory.'
        ' You can resolve this issue by running:'
        '\n\n\tsudo snap connect rclip:removable-media\n\n'
        'This command will grant rclip the necessary access to the "/media" directory.'
        ' Afterward, you can try again.'
      )
      sys.exit(1)
    print(
      'Running rclip outside of the home or "/media" directories is not supported by its snap version.'
      ' If you want to use rclip outside of home or "/media",'
      ' file an issue in the rclip project on GitHub https://github.com/yurijmikhalevich/rclip/issues,'
      ' describe your use case, and consider alternative rclip installation'
      ' options https://github.com/yurijmikhalevich/rclip#linux.'
    )
    sys.exit(1)



import os
from typing import Callable, Pattern

COUNT_FILES_UPDATE_EVERY = 10_000


def count_files(
  directory: str,
  exclude_dir_re: Pattern[str],
  file_re: Pattern[str],
  on_change: Callable[[int], None]
) -> None:
  prev_update_count = 0
  count = 0
  for _ in walk(directory, exclude_dir_re, file_re):
    count += 1
    if count - prev_update_count >= COUNT_FILES_UPDATE_EVERY:
      on_change(count)
      prev_update_count = count
  on_change(count)


def walk(
  directory: str,
  exclude_dir_re: Pattern[str],
  file_re: Pattern[str],
):
  '''Walks through a directory recursively and yields files that match the given regex'''
  dirs_to_process = [directory]
  while dirs_to_process:
    dir = dirs_to_process.pop()
    with os.scandir(dir) as it:
      for entry in it:
        if entry.is_dir():
          if not exclude_dir_re.match(entry.path):
            dirs_to_process.append(entry.path)
        elif entry.is_file() and file_re.match(entry.name):
          yield entry



import base64
from io import BytesIO
import os
from PIL import Image


def _get_start_sequence():
  term_env_var = os.getenv('TERM')
  if term_env_var and (term_env_var.startswith('screen') or term_env_var.startswith('tmux')):
    return '\033Ptmux;\033\033]'
  return '\033]'


def _get_end_sequence():
  term_env_var = os.getenv('TERM')
  if term_env_var and (term_env_var.startswith('screen') or term_env_var.startswith('tmux')):
    return '\a\033\\'
  return '\a'


def preview(filepath: str, img_height_px: int):
  with Image.open(filepath) as img:
    if img_height_px >= img.height:
      width_px, height_px = img.width, img.height
    else:
      width_px, height_px = int(img_height_px * img.width / img.height), img_height_px
    img = img.resize((width_px, height_px), Image.LANCZOS)  # type: ignore
    buffer = BytesIO()
    img.convert('RGB').save(buffer, format='JPEG')
  img_bytes = buffer.getvalue()
  img_str = base64.b64encode(img_bytes).decode('utf-8')
  print(
    f'{_get_start_sequence()}1337;'
    f'File=inline=1;size={len(img_bytes)};preserveAspectRatio=1;'
    f'width={width_px}px;height={height_px}px:{img_str}{_get_end_sequence()}',
  )
